package jp.co.gihyo.webdb.vol35.tdd.webapp.rest.impl;

import java.util.Arrays;
import java.util.List;

import jp.co.gihyo.webdb.vol35.tdd.webapp.rest.DaoOperation;
import jp.co.gihyo.webdb.vol35.tdd.webapp.rest.NoncompliantResourceException;

import org.seasar.framework.beans.BeanDesc;
import org.seasar.framework.beans.factory.BeanDescFactory;
import org.seasar.framework.container.ComponentDef;
import org.seasar.framework.container.S2Container;

public class DaoOperationImpl implements DaoOperation {

    private static final String DAO_COMPONENT_NAME_SUFFIX = "Dao";
    private static final String GET_ALL_METHOD_NAME = "getAll";
    private static final String GET_BY_ID_METHOD_NAME = "getById";
    
    public Object getById(String daoNamePrefix, long id) {
        Object[] args = new Object[]{new Long(id)};
        return invokeDao(daoNamePrefix, GET_BY_ID_METHOD_NAME, args);
    }
    
    public List getAll(String daoNamePrefix) {
        Object entities =
            invokeDao(daoNamePrefix, GET_ALL_METHOD_NAME, null);
        if(entities instanceof List) {
            return (List) entities;
        } else if(entities instanceof Object[]) {
            return Arrays.asList((Object[]) entities);
        } else {
            throw new NoncompliantResourceException(
                    "getAll only supports List or Array");
        }
    }

    private Object invokeDao(
            String daoNamePrefix,
            String methodName,
            Object[] args)
    {
        String daoName = daoNamePrefix + DAO_COMPONENT_NAME_SUFFIX;
        ComponentDef cd =
            container.getComponentDef(daoName);
        BeanDesc desc =
            BeanDescFactory.getBeanDesc(cd.getComponentClass());
        Object entity = desc.invoke(
                cd.getComponent(),
                methodName,
                args);
        return entity;
    }
    
    private S2Container container;
    public void setContainer(S2Container container) {
        this.container = container;
    }
}
