package jp.co.gihyo.webdb.vol35.tdd.webapp.rest.impl;

import java.util.Arrays;
import java.util.List;

import jp.co.gihyo.webdb.vol35.tdd.webapp.rest.NoncompliantResourceException;

import ognl.NoSuchPropertyException;
import ognl.Ognl;

import org.custommonkey.xmlunit.XMLTestCase;
import org.custommonkey.xmlunit.XMLUnit;

public class EntityListMarshallerImplTest extends XMLTestCase {
    EntityListMarshallerImpl marshaller;

    protected void setUp() throws Exception {
        XMLUnit.setIgnoreWhitespace(true);
        marshaller = new EntityListMarshallerImpl();
    }

    static class Sample {
        private long id;
        public Sample(long id) {
            this.id = id;
        }
        public long getId() {
            return id;
        }
        public void setId(long id) {
            this.id = id;
        }
    }

    public void testXgXMLo͂̃C[W() throws Exception {
        List resourceList =
            Arrays.asList(new Sample[]{new Sample(100),new Sample(101)});
        String listUri = "http://localhost/foo/employee";
        String resourceName = "employee";

        String actual = marshaller.marshal(resourceList, resourceName, listUri);

        String expected =
            "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>" +
            "<employeeList xmlns:xlink=\"http://www.w3.org/1999/xlink\">" +
            "  <employee id=\"100\" xlink:href=\"http://localhost/foo/employee/100\"/>" +
            "  <employee id=\"101\" xlink:href=\"http://localhost/foo/employee/101\"/>" +
            "</employeeList>";

        assertXMLEqual(expected, actual);
    }
    
    public void testXgXMLo͂̃C[W̎Op() throws Exception {
        List entityList =
            Arrays.asList(new Sample[]{new Sample(200),new Sample(201)});
        String listUri = "http://localhost/foo/department";
        String resourceName = "department";

        String actual = marshaller.marshal(entityList, resourceName, listUri);

        String expected =
            "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>" +
            "<departmentList xmlns:xlink=\"http://www.w3.org/1999/xlink\">" +
            "  <department id=\"200\" xlink:href=\"http://localhost/foo/department/200\"/>" +
            "  <department id=\"201\" xlink:href=\"http://localhost/foo/department/201\"/>" +
            "</departmentList>";

        assertXMLEqual(expected, actual);
    }
    
    public void testOGNLœIɃvpeB̒l擾@() throws Exception {
        Sample sample = new Sample(300);
        Object value = Ognl.getValue("id", sample);
        assertEquals(new Long(300), value);
    }
    
    public void testOGNL͎擾vpeBꍇɂNoSuchPropertyException𓊂()
            throws Exception {
        try {
            Ognl.getValue("id", new NoId());
            fail();
        } catch (NoSuchPropertyException expected) {
        }
    }
    static class NoId {
    }
    
    
    public void testIdvpeB݂ȂEntitynꂽꍇɂNoncompliantResourceException𓊂() throws Exception {
        List entityList =
            Arrays.asList(new NoId[]{new NoId()});
        String listUri = "http://localhost/foo/noncompliant";
        String resourceName = "noncompliant";

        try {
            marshaller.marshal(entityList, resourceName, listUri);
            fail();
        } catch (NoncompliantResourceException expected) {
        }
    }
}
